#include <Python.h>

static char module_doc[] = 
"This module contains an eclectic selection of functions.";

static char is_even_doc[] =
"is_even(n) -- indicate if the integer n is even";

static char classify_characters_doc[] = 
"classify_characters(str) -- classify the characters in str";

static char set_password_doc[] = 
"set_password(str) -- set the module password to str";

static char get_password_doc[] =
"get_password() -- get the module password";

static char goto_beach_doc[] =
"goto_beach() -- send some poor sole to the beach in Vancouver";

static char logical_xor_doc[] =
"logical_xor(x,y) -- returns 1 if one of x or y is true, 0 otherwise";

static PyObject * WeatherError;

static PyObject *
is_even(PyObject * self, PyObject * args)
{
    long number;

    if (!PyArg_ParseTuple(args, "i", &number))
        return NULL;

    return Py_BuildValue("i", number % 2 == 0);
}


static PyObject *
classify_characters(PyObject * self, PyObject * args)
{
    char * string;
    long length;
    long alpha = 0, other = 0, i;

    if (!PyArg_ParseTuple(args, "s#", &string, &length))
        return NULL;

    for (i = 0; i < length; ++i) {
        isalnum(string[i]) ? ++alpha : ++other;
    }

    return Py_BuildValue("(iif)", alpha, other, ((float) alpha) / length);

}

static PyObject * password = NULL;

static PyObject *
set_password(PyObject * self, PyObject * args)
{
    PyObject * new_password;

    if (!PyArg_ParseTuple(args, "O", &new_password))
        return NULL;

    if (!PyString_Check(new_password) &&
        !PyUnicode_Check(new_password))
    {
        PyErr_Format(PyExc_TypeError, "expected string or "
                    "Unicode , %80s found",
                    new_password->ob_type->tp_name);
        return NULL;
    }

    if (password != NULL)           /* or Py_XDECREF (password) */
        Py_DECREF(password);

    password = new_password;
    Py_INCREF(password);

    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *
get_password(PyObject * self, PyObject * args)
{
    if (!PyArg_ParseTuple(args, ""))
        return NULL;

    if (password == NULL) {
        Py_INCREF(Py_None);
        return Py_None;
    } else {
        Py_INCREF(password);
        return password;
    }
}

static PyObject *
goto_beach(PyObject * self, PyObject * args)
{
    if (!PyArg_ParseTuple(args, ""))
        return NULL;

    if (rand() % 2) {
        PyObject * e = Py_BuildValue("(si)", "too cold", 5);  
        if (e == NULL)
            return NULL;

        PyErr_SetObject(WeatherError, e);
        Py_DECREF(e);
    } else {
        PyErr_SetString(WeatherError, "too rainy");
    }

    return NULL;
}

static PyObject *
logical_xor(PyObject * self, PyObject * args)
{
    PyObject * x;
    PyObject * y;
    long ret;

    if (!PyArg_ParseTuple(args, "OO", &x,&y))
        return NULL;

    ret = PyObject_IsTrue(x) ^ PyObject_IsTrue(y);
    return Py_BuildValue("i", ret);
}

static PyMethodDef module_functions[] = 
{
    {"is_even", is_even, METH_VARARGS, is_even_doc},
    {"set_password", set_password, METH_VARARGS, set_password_doc},
    {"get_password", get_password, METH_VARARGS, get_password_doc},
    {"classify_characters", classify_characters, METH_VARARGS, 
        classify_characters_doc},
    {"goto_beach", goto_beach, METH_VARARGS, goto_beach_doc},
    {"logical_xor", logical_xor, METH_VARARGS, logical_xor_doc},
    {NULL, NULL}
};

#ifdef WIN32
	_declspec(dllexport) 
#endif
void initVanpy(void)
{
    PyObject *  module;
    PyObject *  module_dict;
    
    module = Py_InitModule3("Vanpy", module_functions, module_doc);
    
    WeatherError = PyErr_NewException("Vanpy.WeatherError", NULL, NULL);
    module_dict = PyModule_GetDict(module);
    PyDict_SetItemString(module_dict, "WeatherError", WeatherError);
        
    PyModule_AddStringConstant(module, "__version__", "0.0.1");
}
